/*
 * Copyright (c) 2024. Proton AG
 *
 * This file is part of ProtonVPN.
 *
 * ProtonVPN is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * ProtonVPN is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ProtonVPN.  If not, see <https://www.gnu.org/licenses/>.
 */

package com.protonvpn.android.widget.ui

import android.appwidget.AppWidgetManager
import android.content.Context
import androidx.glance.GlanceId
import androidx.glance.appwidget.GlanceAppWidget
import androidx.glance.appwidget.GlanceAppWidgetManager
import androidx.glance.appwidget.provideContent
import com.protonvpn.android.widget.WidgetStateUpdater
import com.protonvpn.android.widget.hasMaterialYouTheme
import dagger.hilt.EntryPoint
import dagger.hilt.EntryPoints
import dagger.hilt.InstallIn
import dagger.hilt.components.SingletonComponent

class ProtonVpnGlanceWidget : GlanceAppWidget() {

    @EntryPoint
    @InstallIn(SingletonComponent::class)
    interface Dependencies {
        fun getWidgetStateUpdater(): WidgetStateUpdater
    }

    override suspend fun provideGlance(context: Context, id: GlanceId) {
        val entryPoints = EntryPoints.get(context, Dependencies::class.java)
        val manager = GlanceAppWidgetManager(context)
        val widgetInfo = AppWidgetManager.getInstance(context).getAppWidgetInfo(manager.getAppWidgetId(id))

        provideContent {
            ProtonGlanceTheme(isMaterialYou = hasMaterialYouTheme(widgetInfo.provider)) {
                //TODO: Widget content
            }
        }
    }
}
